# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: compression.proto
# Protobuf Python Version: 5.26.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x11\x63ompression.proto\x12\x04kudu*w\n\x0f\x43ompressionType\x12\x18\n\x13UNKNOWN_COMPRESSION\x10\xe7\x07\x12\x17\n\x13\x44\x45\x46\x41ULT_COMPRESSION\x10\x00\x12\x12\n\x0eNO_COMPRESSION\x10\x01\x12\n\n\x06SNAPPY\x10\x02\x12\x07\n\x03LZ4\x10\x03\x12\x08\n\x04ZLIB\x10\x04\x42\x11\n\x0forg.apache.kudu')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'compression_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\017org.apache.kudu'
  _globals['_COMPRESSIONTYPE']._serialized_start=27
  _globals['_COMPRESSIONTYPE']._serialized_end=146
# @@protoc_insertion_point(module_scope)
