syntax = "proto3";

package cz.proto;

message NetworkPolicy {
  NetworkPolicyData     content       = 1;
}

message NetworkPolicyData {
  repeated string       workspaceList = 1;
  repeated string       usernameList  = 2;
  repeated string       blockedList   = 3;
  repeated string       allowedList   = 4;
}
